#!python
from .readme import *
from .contrib import *
from .issues import *
from .pull_requests import *
from .conduct import *
from .gitignore import *
from .feature_requests import *
from .bug_report import *
from .licences import *
from .workflows import *
from .funding import *
from .colors import LanguageColorMap
from .gitattributes import *
from .mailmap import *

if __name__ == "__main__":
    print("Run initgitdoc instead!")



#!python
"""
TralahTek LLC Git Repository Code Documentation ToolKit
Making Code Documentation Less of a Pain in the Ass.
"""
import glob
import os
import sys
from argparse import ArgumentParser
from random import choice
from docrepotek import templates


def resolve_key(lang: str) -> str:
    if lang in templates.LanguageColorMap.keys():
        return lang
    elif (
        lang not in templates.LanguageColorMap.keys()
        and len([x for x in templates.LanguageColorMap.keys() if lang in x]) > 0
    ):
        return choice([x for x in templates.LanguageColorMap.keys() if lang in x])
    else:
        if lang == "cpp":
            return "c++"
        else:
            return "shell"


def resolve_color(language: str) -> str:
    lookup = language.lower()
    key = resolve_key(lookup)
    return templates.LanguageColorMap.get(key, "1110101")


class GitRepo:
    def link_scripts():
        "generate links to files in the github project."
        base_url = "https://github.com/{1}/{0}/blob/master/"
        return "\n\n".join(
            ["* [%s](%s)" % (r, base_url + r)
             for r in glob.glob("*", recursive=True)]
        )

    def gen_RMe(rp_name, rp_author, subdir=False, lang="Python"):
        # scripts = GitRepo.link_scripts().format(rp_name, rp_author)
        color = resolve_color(lang)
        content = templates.readme_template.format(
            rp_name, rp_author, lang, color)
        with open("README.md", "w") as f:
            f.write(content)
        print("Initialized ReadMe.md")

    def gen_Funding(rp_author):
        if not os.path.exists(".github"):
            os.mkdir(".github")
        with open(".github/FUNDING.yml", "w") as f:
            f.write(templates.funding_template.format(rp_author))
        print("Initialized  .github/FUNDING.yml")

    def gen_License():
        content = templates.gpl3
        with open("LICENSE", "w") as f:
            f.write(content)
        print("Initialized LICENSE")

    def gen_Contrib():
        content = templates.contributing_template
        with open("CONTRIBUTING.rst", "w") as f:
            f.write(content)
        print("Initialized CONTRIBUTING.rst")

    def gen_Conduct():
        with open("CODE_OF_CONDUCT.md", "w") as f:
            f.write(templates.code_of_conduct)
        print("Initialized CODE_OF_CONDUCT.md")

    def gen_Issues_PR():
        if not os.path.exists(".github"):
            os.mkdir(".github")
        if not os.path.exists(".github/ISSUE_TEMPLATE"):
            os.mkdir(".github/ISSUE_TEMPLATE")
        with open(".github/ISSUE_TEMPLATE/bug_report.md", "w") as f:
            f.write(templates.bug_report)
        print("Initialized .github/ISSUE_TEMPLATE/bug_report.md")
        with open(".github/ISSUE_TEMPLATE/feature_request.md", "w") as f:
            f.write(templates.feature_request)
        print("Initialized .github/ISSUE_TEMPLATE/feature_request.md")
        with open("PULL_REQUEST_TEMPLATE.md", "w") as f:
            f.write(templates.pr_template)
        print("Initialized PULL_REQUEST_TEMPLATE.md")

    def gen_Gitignore():
        with open(".gitignore", "w") as f:
            f.write(templates.gitignore)
        print("Initialized .gitignore")

    def gen_Gitattributes():
        with open(".gitattributes", "w") as f:
            f.write(templates.gitattributes)
        print("Initialized .gitattributes")

    def gen_mailmap():
        with open(".mailmap", "w") as f:
            f.write(templates.mailmap)
        print("Created .mailmap")

    def gen_pypi_workflow():
        if not os.path.exists(".github"):
            os.mkdir(".github")
        if not os.path.exists(".github/workflows"):
            os.mkdir(".github/workflows")
        with open(".github/workflows/pythonpublish.yml", "w") as f:
            f.write(templates.pypi_workflow)
        print("Initialized .github/workflows/pythonpublish.yml")
        pass


class Docrepo:
    def __init__(self):
        epilog = "Author: TralahM\n Email: <briantralah@gmail.com>. Copyright:2019 (All Rights Reserved). "
        parser = ArgumentParser(
            epilog=epilog,
            description="Repository Documenter and Template Scaffolder",
            usage="""docrepo <command> [<args>]

            The commonly used docrepo commands are:
            init    Initializes Repository with all common repository files
            readme  Initializes a Repository ReadMe
            license Creates a New LICENSE
            pypi_workflow Create a Python Package Publish Workflow
            """,
        )
        parser.add_argument("command", help="Subcommand to run")
        args = parser.parse_args(sys.argv[1:2])
        if not hasattr(self, args.command):
            print("Unrecognized command")
            parser.print_help()
        getattr(self, args.command)()

    def init(self):
        parser = ArgumentParser(
            description="Initial all documents for this repository")
        parser.add_argument(
            "--author", action="store", help="github username", default="TralahM"
        )
        parser.add_argument(
            "-l",
            "--language",
            action="store",
            dest="lang",
            help="Main Language of the Repository",
            default="Python",
        )
        parser.add_argument(
            "-n",
            "--name-repo",
            action="store",
            dest="name",
            help="The name of your github repository. Default is the name of the current directory",
            default=os.path.abspath(".").split("/")[-1],
        )
        parser.add_argument(
            "-s",
            "--sub-directory",
            action="store_true",
            dest="sub",
            default=False,
            help="Is folder already inside a repo, is it a subdirectory ? default=False",
        )
        parser.add_argument(
            "-nr",
            "-no-readme",
            action="store_true",
            dest="noreadme",
            default=False,
            help="Dont generate a ReadMe default=False",
        )
        parser.add_argument(
            "-pw",
            "-pypi-workflow",
            action="store_true",
            dest="pypi_workflow",
            default=False,
            help="Generate a Pypi Workflow default=False",
        )
        args = parser.parse_args(sys.argv[2:])
        if not args.noreadme:
            GitRepo.gen_RMe(args.name, args.author, args.sub, lang=args.lang)
        if not args.sub:
            GitRepo.gen_Contrib()
            GitRepo.gen_Conduct()
            GitRepo.gen_License()
            GitRepo.gen_Issues_PR()
            GitRepo.gen_Gitignore()
            GitRepo.gen_Funding(args.author)
            GitRepo.gen_Gitattributes()
            GitRepo.gen_mailmap()
        if args.pypi_workflow:
            GitRepo.gen_pypi_workflow()


def main():
    epilog = "Author: TralahM\n Email: <briantralah@gmail.com>. Copyright:2019 (All Rights Reserved). "
    ps = ArgumentParser(
        description="Generate Common Documentation files for any git repository.\n",
        epilog=epilog,
    )
    ps.add_argument(
        "-n",
        "--name-repo",
        action="store",
        dest="name",
        help="The name of your github repository. Default is the name of the current directory",
        default=os.path.abspath(".").split("/")[-1],
    )
    ps.add_argument(
        "-a",
        "--author",
        action="store",
        dest="author",
        help="Your github username, default is TralahM",
        default="TralahM",
    )
    ps.add_argument(
        "-l",
        "--language",
        action="store",
        dest="lang",
        help="Main Language of the Repository",
        default="Python",
    )
    ps.add_argument(
        "-s",
        "--sub-directory",
        action="store_true",
        dest="sub",
        default=False,
        help="Is folder already inside a repo, is it a subdirectory ? default=False",
    )
    ps.add_argument(
        "-nr",
        "-no-readme",
        action="store_true",
        dest="noreadme",
        default=False,
        help="Dont generate a ReadMe default=False",
    )
    ps.add_argument(
        "-pw",
        "-pypi-workflow",
        action="store_true",
        dest="pypi_workflow",
        default=False,
        help="Generate a Pypi Workflow default=False",
    )
    psd = ps.parse_args()
    name, author, sub, lang = psd.name, psd.author, psd.sub, psd.lang
    if sub:
        cd = os.environ["PWD"]
        nname = cd.split(name)[-1]
        name = name + nname
    if not psd.noreadme:
        GitRepo.gen_RMe(name, author, sub, lang=lang)
    if not sub:
        GitRepo.gen_Contrib()
        GitRepo.gen_Conduct()
        GitRepo.gen_License()
        GitRepo.gen_Issues_PR()
        GitRepo.gen_Gitignore()
        GitRepo.gen_Funding(author)
        GitRepo.gen_Gitattributes()
        GitRepo.gen_mailmap()
    if psd.pypi_workflow:
        GitRepo.gen_pypi_workflow()


if __name__ == "__main__":
    main()



mailmap = """
# Prevent git from showing duplicate names with commands like "git shortlog"
# See the manpage of git-shortlog for details.
# The syntax is:
# Name that should be used <email that should be used> Bad name <bad email>
#
# You can skip Bad name if it is the same as the one that should be used, and is unique.
#
# This file is up-to-date if the command git log --format="%aN <%aE>" | sort -u
# gives no duplicates.

Tralah M Brian <musyoki.brian@tralahtek.com> TralahM <musyoki.brian@tralahtek.com>

"""



code_of_conduct = """
# Contributor Covenant Code of Conduct

## Our Pledge

In the interest of fostering an open and welcoming environment, we as
contributors and maintainers pledge to making participation in our project and
our community a harassment-free experience for everyone, regardless of age, body
size, disability, ethnicity, sex characteristics, gender identity and expression,
level of experience, education, socio-economic status, nationality, personal
appearance, race, religion, or sexual identity and orientation.

## Our Standards

Examples of behavior that contributes to creating a positive environment
include:

* Using welcoming and inclusive language
* Being respectful of differing viewpoints and experiences
* Gracefully accepting constructive criticism
* Focusing on what is best for the community
* Showing empathy towards other community members

Examples of unacceptable behavior by participants include:

* The use of sexualized language or imagery and unwelcome sexual attention or
 advances
* Trolling, insulting/derogatory comments, and personal or political attacks
* Public or private harassment
* Publishing others' private information, such as a physical or electronic
 address, without explicit permission
* Other conduct which could reasonably be considered inappropriate in a
 professional setting

## Our Responsibilities

Project maintainers are responsible for clarifying the standards of acceptable
behavior and are expected to take appropriate and fair corrective action in
response to any instances of unacceptable behavior.

Project maintainers have the right and responsibility to remove, edit, or
reject comments, commits, code, wiki edits, issues, and other contributions
that are not aligned to this Code of Conduct, or to ban temporarily or
permanently any contributor for other behaviors that they deem inappropriate,
threatening, offensive, or harmful.

## Scope

This Code of Conduct applies both within project spaces and in public spaces
when an individual is representing the project or its community. Examples of
representing a project or community include using an official project e-mail
address, posting via an official social media account, or acting as an appointed
representative at an online or offline event. Representation of a project may be
further defined and clarified by project maintainers.

## Enforcement

Instances of abusive, harassing, or otherwise unacceptable behavior may be
reported by contacting the project team at musyoki.tralah@students.jkuat.ac.ke. All
complaints will be reviewed and investigated and will result in a response that
is deemed necessary and appropriate to the circumstances. The project team is
obligated to maintain confidentiality with regard to the reporter of an incident.
Further details of specific enforcement policies may be posted separately.

Project maintainers who do not follow or enforce the Code of Conduct in good
faith may face temporary or permanent repercussions as determined by other
members of the project's leadership.

## Attribution

This Code of Conduct is adapted from the [Contributor Covenant][homepage], version 1.4,
available at https://www.contributor-covenant.org/version/1/4/code-of-conduct.html

[homepage]: https://www.contributor-covenant.org

For answers to common questions about this code of conduct, see
https://www.contributor-covenant.org/faq
"""



contributing_template = """
=======================
Contributing Guidelines
=======================

Thanks for your interest in contributing to this project!  Contributions are
welcome and very much appreciated.  If this is your first time contributing to a
Free and Open Source Software project, consider reading `How to Contribute to
Open Source`_ in the Open Source Guides.  Additionally, to maximize the chance
that your contribution will be accepted and minimize wasted effort, consider the
following guidelines:


Tests Must Pass
===============

In order for a pull request to be accepted, it must build and test successfully
on the continuous integration systems.  GitHub displays a build status
indicator on the pull request.  If the build is broken, please fix it (or add a
comment if you think the pull request is not the cause of the breakage).

The CI build runs several linters and tests.  These can be run locally by
invoking ``tox``.  Disabling linter rules using inline configuration is
acceptable, where justified.  Disable specific rules where possible (i.e.
``# noqa: F401`` rather than just ``# noqa``).


Add Tests Where Reasonable
==========================

If the pull request fixes a bug or adds a feature, consider adding a test to
ensure the bug does not reoccur and the feature works as expected.  If testing
is not straightforward, feel free to submit the pull request without tests.
How to test and whether testing is required can be discussed during the review
process.


Consider Discussing Big Changes First
=====================================

If the desired change is large, complex, backwards-incompatible, can have
significantly differing implementations, or may not be in scope for this
project, opening an issue to discuss the change before writing the code can
avoid frustration and save a lot of time and effort.

This is not a hard requirement.  If you'd rather start discussing a big change
with a proposed implementation, feel free.  Be aware that the code may be
rejected outright, or require many changes before it is acceptable.

.. _How to Contribute to Open Source: https://opensource.guide/how-to-contribute/
"""
